/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var Annotate = function()
{
	var annotate = {};

	var annotations;
	var selectedAnnotation;
	var selectedIndex;
	
	/**
		Initialize all input fields were we want to detect changes to values. Most fields react to
		the 'change' event, but radio-buttons and check-boxes react to the 'click' event.
	*/
	annotate.initSingleValues = function(element, autoInit)
	{
		if (autoInit == 'detect-change')
		{
			var eventType = element.type == 'radio' || element.type == 'checkbox' ? 'click' : 'change';
			Events.addEventHandler(element, eventType, annotate.valueOnChange);
		}
	}
	Doc.addElementInitializer(annotate.initSingleValues);
	
	annotate.initPage = function()
	{
		// Buttons (on standalone dialog)
		Buttons.addClickHandler('close', App.closeWindow);
		Buttons.addClickHandler('btnSave', annotate.save);
		
		// Buttons for multi-valued annotations
		Buttons.addClickHandler('btnMultiAdd', annotate.addMultiValue);
		Buttons.addClickHandler('btnMultiRemove', annotate.removeMultiValue);
		Events.addEventHandler('multi-values', 'click', annotate.multiValueOnClick);
		
		// Parse the current annotation information which contain value type, current values, etc.
		annotations = Data.json('page-data', 'annotations');
		
		// The category list box is used to filter visible annotations
		Events.addEventHandler('categories', 'change', annotate.categoryOnChange);
		annotate.categoryOnChange();
		
		// Loop through all annotation...
		for (var i = 0; i < annotations.length; i++)
		{
			var annotation = annotations[i];
			annotations['ID'+annotation.id] = annotation;
			// set the icon
			annotate.updateIcon(annotation);
			// add event handler for selecting the annotation
			Events.addEventHandler('prompt.'+annotation.id, 'click', annotate.annotationOnClick);
			
			// Attach event handler that are specific to certain value types
			var valueType = annotation.valueType;
			var fieldName = 'value.'+annotation.id;
			if (!annotation.enumeration)
			{
				if (valueType == 'INT' || valueType == 'LONG')
				{
					// Block non-numeric keys from INT and LONG fields
					Events.addEventHandler(fieldName, 'keypress', Events.integerOnly);
				}
				else if (valueType == 'FLOAT' || valueType == 'DOUBLE')
				{
					// Block non-numeric keys from FLOAT and DOUBLE fields
					Events.addEventHandler(fieldName, 'keypress', Events.numberOnly);
				}
				else if (valueType == 'DATE' || valueType == 'TIMESTAMP')
				{
					// Add click handler to open calendar dialog for date/timestamp fields
					Buttons.addClickHandler('btn.'+annotation.id, annotate.openCalendar);
				}
				if (annotation.unit)
				{
					Events.addEventHandler('unit.'+annotation.id, 'change', annotate.unitOnChange);
				}
			}
		}
		
		// Select the initial annotation
		var annotationTypeId = Data.int('page-data', 'annotation-type-id');
		if (annotationTypeId) annotate.selectAnnotation(annotationTypeId)
	}
	
	
	/**
		Event handler for the 'Categories' list. This will
		filter the main list to only include annotations in the select
		category. Special cases are 'all', 'protocol parameter' and 'uncategorized'
	 */
	annotate.categoryOnChange = function()
	{
		var frm = document.forms['annotations'];
		var categoryId = parseInt(frm.categories.value);
		for (var i = 0; i < annotations.length; i++)
		{
			var annotation = annotations[i];
			var show = false;
			if (categoryId == -1)
			{
				// show all
				show = true;
			}
			else if (categoryId == -2 && annotation.parameter)
			{
				// show protocol parameters
				show = true;
			}
			else if (categoryId == 0 && annotation.categories.length == 0 && !annotation.parameter)
			{
				// show uncategorized
				show = true;
			}
			else
			{
				// show the selected category
				for (var j = 0; j < annotation.categories.length; j++)
				{
					if (annotation.categories[j] == categoryId) show = true;
				}
			}
			Doc.showHide('prompt.'+annotation.id, show);
		}
	}
	
	/**
		Select the given annotation. This will mark the annotation in the main list
		and display the input fields associated with it.
	*/
	annotate.selectAnnotation = function(annotationTypeId)
	{
		if (selectedAnnotation)
		{
			// Hide current div
			Doc.hide('section.'+selectedAnnotation.id);
			Doc.removeClass('prompt.'+selectedAnnotation.id, 'selected');
		}
		selectedAnnotation = annotations['ID'+annotationTypeId];
		selectedIndex = -1;
		Doc.show('section.'+annotationTypeId);
		Doc.addClass('prompt.'+annotationTypeId, 'selected');

		var frm = document.forms['annotations'];
		var valueElement = frm['value.'+annotationTypeId];
		if (valueElement.focus) valueElement.focus();

		if (selectedAnnotation.enumeration)
		{
			Doc.hide('valuecontainer');
			if (valueElement.type && valueElement.type.indexOf('select') == 0)
			{
				// selection list
				Forms.selectListOptions(valueElement, selectedAnnotation.values);
			}
			else if (selectedAnnotation.multiplicity == 1)
			{
				// radio buttons
				var value = selectedAnnotation.values.length > 0 ? selectedAnnotation.values[0] : '';
				Forms.checkRadio(valueElement, value);
			}
			else
			{
				// check boxes
				Forms.checkCheckBoxes(valueElement, selectedAnnotation.values);
			}
		}
		else if (selectedAnnotation.multiplicity == 1)
		{
			Doc.hide('valuecontainer');
			if (selectedAnnotation.values.length > 0)
			{
				if (valueElement.length) // ie. radio buttons for boolean parameter
				{
					Forms.checkRadio(valueElement, selectedAnnotation.values[0]);
				}
				else
				{
					valueElement.value = selectedAnnotation.values[0];
				}
			}
		}
		else
		{
			var multiValues = frm['multi-values'];
			multiValues.length = 0;
			for (var i = 0; i < selectedAnnotation.values.length; i++) 
			{
				multiValues[multiValues.length] = new Option(selectedAnnotation.values[i]);
			}
			var mult = Doc.element('multiplicity');
			mult.innerHTML = selectedAnnotation.multiplicity == 0 ? '' : '(Max '+selectedAnnotation.multiplicity+' values)';
			Doc.show('valuecontainer');
		}
	}
	
	/**
		Event handler for selecting annotations from the main list.
	*/
	annotate.annotationOnClick = function(event)
	{
		var annotationTypeId = Data.int(event.currentTarget, 'annotation-type-id');
		annotate.selectAnnotation(annotationTypeId);
	}
	
	/**
		Update the given annotation with a single value. All existing
		values are overwritten. If the value is not accepted an
		exception is thrown.
	*/
	annotate.setSingleValue = function(annotation, value)
	{
		var values = [];
		if (value != null && value != '')
		{
			values[0] = value;
		}
		annotate.setMultipleValues(annotation, values);
	}
	
	/**
		Update the given annotation with multiple values. All existing
		values are overwritten. If the values are not accepted an
		exception is thrown.
	*/
	annotate.setMultipleValues = function(annotation, values)
	{
		if (annotation.multiplicity > 0 && values.length > annotation.multiplicity)
		{
			throw 'Max '+annotation.multiplicity+' values allowed';
		}
		annotation.values = values;
		annotation.modified = true;
		annotate.updateIcon(annotation);
	}
	
	/**
		Update/add/remove a single value in a multi-valued annotation. If the value
		is not accepted an exception is thrown.
	*/
	annotate.setIndexedValue = function(annotation, index, value)
	{
		if (!value)
		{
			if (index < annotation.values.length)
			{
				annotation.values.splice(index, 1);
			}
		}
		else
		{
			if (index >= annotation.values.length)
			{
				// Add a new value
				if (annotation.multiplicity > 0 && annotation.values.length >= annotation.multiplicity)
				{
					throw 'Max '+annotation.multiplicity+' values allowed';
				}
				index = annotation.values.length;
			}
			annotation.values[index] = value;
		}
		annotation.modified = true;
		annotate.updateIcon(annotation);
	}
	
	/**
		Change the unit of the given annotation.
	*/
	annotate.setUnit = function(annotation, unit)
	{
		annotation.unit = unit;
		annotation.modified = true;
	}
	
	/**
		Event handler that is executed when an annotation values has changed.
		This new value should be stored back into the 'annotations' array.
		The exact procedure depends on the type of value.
	*/
	annotate.valueOnChange = function(event)
	{
		if (!selectedAnnotation) return;
		
		var frm = document.forms['annotations'];
		var target = event.currentTarget;
		try
		{
			if (selectedAnnotation.multiplicity == 1) 
			{
				// If only a single value is accepted the event target holds the new value
				annotate.setSingleValue(selectedAnnotation, target.value);
			}
			else if (selectedAnnotation.enumeration)
			{
				// If the annotation is a multi-valued enumeration the input fields can be either a
				// group of checkboxes or a select control allowing multiple selections.
				if (target.type == 'checkbox') target = frm[target.name]; // An array of all checkboxes in the group
				var values = [];
				for (var i = 0; i < target.length; i++)
				{
					if (target[i].checked || target[i].selected) 
					{
						values[values.length] = target[i].value;
					}
				}
				annotate.setMultipleValues(selectedAnnotation, values);
			}
			else
			{
				// Otherwise we use the 'multi-values' control and update the currently
				// selected option
				if (selectedIndex >= 0)
				{
					annotate.setIndexedValue(selectedAnnotation, selectedIndex, target.value);
					var multiValues = frm['multi-values'];
					if (!target.value)
					{
						multiValues[selectedIndex] = null;
						selectedIndex = -1;
					}
					else
					{
						multiValues[selectedIndex].text = target.value;
					}
				}
			}
		}
		catch (errorMessage)
		{
			event.preventDefault();
			Forms.showNotification(event.currentTarget, errorMessage);
		}
	}
	
	/**
		Event handler that handle changes to the selected unit.
	*/
	annotate.unitOnChange = function(event)
	{
		annotate.setUnit(selectedAnnotation, event.currentTarget.value);
	}

	annotate.addMultiValue = function(event)
	{
		if (!selectedAnnotation) return;
		
		var frm = document.forms['annotations'];
		var valueElement = frm['value.'+selectedAnnotation.id];
		var multiValues = frm['multi-values'];
		var errorMessage = null;
		try
		{
			if (selectedIndex >= 0)
			{
				// Add a new option
				errorMessage = annotate.setIndexedValue(selectedAnnotation, multiValues.length, '');
				if (!errorMessage)
				{
					multiValues[multiValues.length] = new Option('<new>', '', false, true);
					selectedIndex = multiValues.length-1;
					valueElement.value = '';
				}
			}
			else if (valueElement.value != '')
			{
				// Add the current value
				errorMessage = annotate.setIndexedValue(selectedAnnotation, multiValues.length, valueElement.value);
				if (!errorMessage)
				{
					multiValues[multiValues.length] = new Option(valueElement.value, '', false, false);
					valueElement.value = '';
				}
			}
			valueElement.focus();
			annotate.updateIcon(selectedAnnotation);
		}
		catch (errorMessage)
		{
			Forms.showNotification('btnMultiAdd', errorMessage, null, 'pointer-left');
		}
	}
	
	annotate.removeMultiValue = function()
	{
		if (!selectedAnnotation || selectedIndex < 0) return;
		
		var frm = document.forms['annotations'];
		var multiValues = frm['multi-values'];
		
		annotate.setIndexedValue(selectedAnnotation, multiValues.selectedIndex, null);
		multiValues[multiValues.selectedIndex] = null;
		frm['value.'+selectedAnnotation.id].value = '';
		frm['value.'+selectedAnnotation.id].focus();
	}
	
	/**
		Event handler for the 'multi-values' selection list. This should change
		the value that is currently being edited to the selected option.
	*/
	annotate.multiValueOnClick = function()
	{
		if (!selectedAnnotation) return;
		var frm = document.forms['annotations'];
		selectedIndex = frm['multi-values'].selectedIndex;
		if (selectedIndex >= 0)
		{
			frm['value.'+selectedAnnotation.id].value = selectedAnnotation.values[selectedIndex];
			frm['value.'+selectedAnnotation.id].focus();
		}
	}
	
	/**
		Update the icon for the annotation. The icon is different depending on if
		there are values or not for the annotation.
	*/
	annotate.updateIcon = function(annotation)
	{
		var icon;
		if (annotation.values.length == 0)
		{
			icon = annotation.required ? 'required_novalues.gif' : 'notrequired_novalues.gif';
		}
		else
		{
			icon = annotation.required ? 'required_values.gif' : 'notrequired_values.gif';
		}
		var img = Doc.element('icon.'+annotation.id);
		img.src = App.getRoot()+'images/'+icon;
	}
	
	/**
		Open the calendar dialog for the selected annotation.
	*/ 
	annotate.openCalendar = function(event)
	{
		var textarea = 'value.'+selectedAnnotation.id;
		var title = selectedAnnotation.name;
		var useTime = selectedAnnotation.valueType == 'TIMESTAMP';
		var format = Data.get('page-data', useTime ? 'datetime-format' : 'date-format');
		Dialogs.openCalendar(textarea, title, format, useTime);
	}

	/**
		Copy all modified annotation values to the given form as hidden fields.
	*/
	annotate.saveModifiedAnnotationToForm = function(frm)
	{
		var modified = [];
		for (var i = 0; i < annotations.length; i++)
		{
			var annotation = annotations[i];
			if (annotation.modified)
			{
				modified[modified.length] = annotation.id;
				for (var j = 0; j < annotation.values.length; j++)
				{
					Forms.addHidden(frm, annotation.id, annotation.values[j]);
				}
				if (annotation.unit) Forms.addHidden(frm, 'unit:'+annotation.id, annotation.unit);
			}
		}
		if (frm.modifiedAnnotations)
		{
			frm.modifiedAnnotations.value = modified.join(',');
		}
		else
		{
			Forms.addHidden(frm, 'modifiedAnnotations', modified.join(','));
		}
	}
	
	/**
		Save the annotations in standalone mode.
	*/
	annotate.save = function()
	{
		var frm = document.forms['modified'];
		annotate.saveModifiedAnnotationToForm(frm);
		frm.submit();
	}
	
	return annotate;
}();

Doc.onLoad(Annotate.initPage);


